/****************************************************************************
  File: qtools.c
  Module:  AzaeaQTools

  Prototypes:
    void AzaleaQTools (BYTE *outbuf, BYTE *inbuf, WORD insize);

  Revision History:
    14Sep00  Cory R. King (coryking@azalea.com)
        Created based on Azalea Software's existing barcode library and sample
        code.

  Description:  Creates generic QCode font strings in outbuf,
        for use with Azalea Software's Code128 Font, given a Null-terminated
        ASCII string in inbuf, and the count of bytes in inbuf, in insize.

        Note that the insize must be even; an empty
        string (0-length) will be returned if insize is odd.

  Constraints:  Does not enforce valid data. It's the caller's responsibility
        to select the correct code set for the data to be encoded.

        Valid data set:
            pairs of numbers, 00..99 (ASCII 48,48..57,57)

  Note: This release does not support embedded Function codes (FNC1, ...).

  This code is held under a GNU licence.  C128Tools is a registered trademark
  of Azalea Software, Inc.

****************************************************************************/
#include "platform.h"

// Prototype the 'general purpose' routine:
void FormatQ(BYTE *outbuf, BYTE *inbuf, WORD insize);

/************************************************************
Public Function
************************************************************/

void LIB_PROC
AzaleaQTools (BYTE *outbuf, BYTE *inbuf, WORD insize)
{
    // assume all-numeric (caller's responsibility), but ensure even length
    if ((insize % 2) != 0)
        *outbuf = '\0';
    else
         FormatQ(outbuf, inbuf, insize);
}
/****************************************************************************
Low-level formatter.
****************************************************************************/
// ASCII char offsets
#define SPACE    32
#define RUBOUT  127
#define ATSIGN   64

void FormatQ(BYTE *outbuf, BYTE *inbuf, WORD insize)
{
    extern BYTE SymbolToFont(BYTE symbol);
    extern char Q_STOP;
    extern char Q_START;

    BYTE    symbol;         // input char work buffer
    BYTE    next_digit;     // temp for codeset C
    WORD    count;          // of input chars processed
    long    checkdigit=0;   // accumulator; can get very big, very fast
    WORD    weight=0;       // weighted input position, for check digit calc.

    // map the start char and output it
    *outbuf++ = SymbolToFont(Q_START);


    // initialize the checkdigit
    checkdigit = Q_START;
    weight = 1;

    // process the input stream
    for (count = 0; count < insize; )
    {
        // get the raw input char
        symbol = *inbuf;

        // compress two input digits to one Symbol char
        symbol -= '0';
        next_digit = *(inbuf +1) - '0';
        // pack 'em
        symbol = (symbol * 10) + next_digit;

        // process next pair
        inbuf += 2;
        count += 2;

        // map symbol char to font char and output it
        *outbuf++ = SymbolToFont(symbol);

        checkdigit += symbol * weight;
        
        weight++;

    }

    // done processing; now finalize the MOD 103 checkdigit
    *outbuf++ = SymbolToFont((BYTE)(checkdigit % 103));

    // place the stop bar and terminate the string
    *outbuf++ = SymbolToFont(Q_STOP);
    *outbuf   = '\0';

}

/* end qtools.c  */


